@@ -2,7 +2,7 @@
 #define CHARACTERINFODIALOG_H
 
 #include <t3widget/widget.h>
-using namespace t3_widget;
+using namespace t3widget;
 
 class character_details_dialog_t : public dialog_t {
  private:
