@@ -30,7 +30,7 @@
 
 #define CREATE_MODE (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH)
 
-file_buffer_t::file_buffer_t(const char *_name, const char *_encoding)
+file_buffer_t::file_buffer_t(string_view _name, string_view _encoding)
     : text_buffer_t(new file_line_factory_t(this)),
       view_parameters(new edit_window_t::view_parameters_t()),
       has_window(false),
@@ -39,23 +39,22 @@ file_buffer_t::file_buffer_t(const char *_name, const char *_encoding)
       match_line(nullptr),
       last_match(nullptr),
       matching_brace_valid(false) {
-  if (_encoding == nullptr || strlen(_encoding) == 0) {
+  if (_encoding.size() == 0) {
     encoding = "UTF-8";
   } else {
-    encoding = _encoding;
+    encoding = std::string(_encoding);
   }
 
-  if (_name == nullptr || strlen(_name) == 0) {
+  if (_name.size() == 0) {
     name_line.set_text("(Untitled)");
   } else {
-    name = _name;
+    name = std::string(_name);
 
-    std::string converted_name;
-    convert_lang_codeset(&name, &converted_name, true);
-    name_line.set_text(&converted_name);
+    std::string converted_name = convert_lang_codeset(name, true);
+    name_line.set_text(converted_name);
   }
 
-  connect_rewrap_required(signals::mem_fun(this, &file_buffer_t::invalidate_highlight));
+  connect_rewrap_required(bind_front(&file_buffer_t::invalidate_highlight, this));
 
   view_parameters->set_tabsize(option.tabsize);
   view_parameters->set_wrap(option.wrap ? wrap_type_t::WORD : wrap_type_t::NONE);
@@ -74,11 +73,10 @@ file_buffer_t::~file_buffer_t() {
 }
 
 rw_result_t file_buffer_t::load(load_process_t *state) {
-  const std::string *line;
   t3_highlight_t *highlight = nullptr;
   t3_highlight_lang_t lang;
   t3_bool success = t3_false;
-  int i;
+  text_pos_t i;
 
   if (state->file != this) {
     PANIC();
@@ -87,12 +85,10 @@ rw_result_t file_buffer_t::load(load_process_t *state) {
   switch (state->state) {
     case load_process_t::INITIAL_MISSING_OK:
     case load_process_t::INITIAL: {
-      std::string converted_name;
-      std::string _name;
       transcript_t *handle;
       transcript_error_t error;
 
-      _name = canonicalize_path(name.c_str());
+      std::string _name = canonicalize_path(name.c_str());
       if (_name.empty()) {
         if (errno == ENOENT && state->state == load_process_t::INITIAL_MISSING_OK) {
           break;
@@ -101,8 +97,8 @@ rw_result_t file_buffer_t::load(load_process_t *state) {
       }
 
       name = _name;
-      convert_lang_codeset(&name, &converted_name, true);
-      name_line.set_text(&converted_name);
+      std::string converted_name = convert_lang_codeset(name, true);
+      name_line.set_text(converted_name);
 
       if ((state->fd = open(name.c_str(), O_RDONLY)) < 0) {
         if (errno == ENOENT && state->state == load_process_t::INITIAL_MISSING_OK) {
@@ -144,7 +140,8 @@ rw_result_t file_buffer_t::load(load_process_t *state) {
               /* FALLTHROUGH */
               case load_process_t::REMOVE_BOM:
                 try {
-                  append_text(state->wrapper->get_buffer() + 3, state->wrapper->get_fill() - 3);
+                  append_text(string_view(state->wrapper->get_buffer() + 3,
+                                          state->wrapper->get_fill() - 3));
                   state->buffer_used = true;
                 } catch (...) {
                   return rw_result_t(rw_result_t::ERRNO_ERROR, ENOMEM);
@@ -158,14 +155,13 @@ rw_result_t file_buffer_t::load(load_process_t *state) {
           }
 
           try {
-            append_text(state->wrapper->get_buffer(), state->wrapper->get_fill());
+            append_text(string_view(state->wrapper->get_buffer(), state->wrapper->get_fill()));
             state->buffer_used = true;
           } catch (...) {
             return rw_result_t(rw_result_t::ERRNO_ERROR, ENOMEM);
           }
         }
-        cursor.pos = 0;
-        cursor.line = 0;
+        set_cursor({0, 0});
       } catch (rw_result_t &result) {
         return result;
       }
@@ -186,19 +182,19 @@ rw_result_t file_buffer_t::load(load_process_t *state) {
      any autodetection based on the first line or the file name.
   */
   for (i = 0; i < size() && i < 5 && !success; i++) {
-    line = get_line_data(i)->get_data();
+    const std::string &line = get_line_data(i).get_data();
     success =
-        t3_highlight_detect(line->data(), line->size(), false, T3_HIGHLIGHT_UTF8, &lang, nullptr);
+        t3_highlight_detect(line.data(), line.size(), false, T3_HIGHLIGHT_UTF8, &lang, nullptr);
   }
   for (i = size() - 1; i >= 5 && !success; i--) {
-    line = get_line_data(i)->get_data();
+    const std::string &line = get_line_data(i).get_data();
     success =
-        t3_highlight_detect(line->data(), line->size(), false, T3_HIGHLIGHT_UTF8, &lang, nullptr);
+        t3_highlight_detect(line.data(), line.size(), false, T3_HIGHLIGHT_UTF8, &lang, nullptr);
   }
   if (!success) {
-    line = get_line_data(0)->get_data();
+    const std::string &line = get_line_data(0).get_data();
     success =
-        t3_highlight_detect(line->data(), line->size(), true, T3_HIGHLIGHT_UTF8, &lang, nullptr);
+        t3_highlight_detect(line.data(), line.size(), true, T3_HIGHLIGHT_UTF8, &lang, nullptr);
   }
   if (!success) {
     success = t3_highlight_lang_by_filename(name.c_str(), T3_HIGHLIGHT_UTF8, &lang, nullptr);
@@ -336,17 +332,16 @@ rw_result_t file_buffer_t::save(save_as_process_t *state) {
         state->state = save_as_process_t::WRITING;
       }
     case save_as_process_t::WRITING: {
-      if (strip_spaces.is_valid() ? strip_spaces() : option.strip_spaces) {
+      if (strip_spaces.is_valid() ? strip_spaces.value() : option.strip_spaces) {
         do_strip_spaces();
       }
       try {
         for (; state->i < size(); state->i++) {
-          const std::string *data;
           if (state->i != 0) {
             state->wrapper->write("\n", 1);
           }
-          data = get_line_data(state->i)->get_data();
-          state->wrapper->write(data->data(), data->size());
+          const std::string &data = get_line_data(state->i).get_data();
+          state->wrapper->write(data.data(), data.size());
         }
       } catch (rw_result_t error) {
         return error;
@@ -375,10 +370,9 @@ rw_result_t file_buffer_t::save(save_as_process_t *state) {
       }
 
       if (!state->name.empty()) {
-        std::string converted_name;
         name = state->name;
-        convert_lang_codeset(&name, &converted_name, true);
-        name_line.set_text(&converted_name);
+        std::string converted_name = convert_lang_codeset(name, true);
+        name_line.set_text(converted_name);
       }
       set_undo_mark();
       break;
@@ -399,16 +393,16 @@ const edit_window_t::view_parameters_t *file_buffer_t::get_view_parameters() con
   return view_parameters.get();
 }
 
-void file_buffer_t::prepare_paint_line(int line) {
-  int i;
+void file_buffer_t::prepare_paint_line(text_pos_t line) {
+  text_pos_t i;
 
   if (highlight_info == nullptr || highlight_valid >= line) {
     return;
   }
 
   for (i = highlight_valid >= 0 ? highlight_valid + 1 : 1; i <= line; i++) {
-    int state = static_cast<file_line_t *>(get_line_data_nonconst(i - 1))->get_highlight_end();
-    static_cast<file_line_t *>(get_line_data_nonconst(i))->set_highlight_start(state);
+    int state = static_cast<file_line_t *>(get_mutable_line_data(i - 1))->get_highlight_end();
+    static_cast<file_line_t *>(get_mutable_line_data(i))->set_highlight_start(state);
   }
   highlight_valid = line;
   match_line = nullptr;
@@ -418,7 +412,7 @@ void file_buffer_t::set_has_window(bool _has_window) { has_window = _has_window;
 
 bool file_buffer_t::get_has_window() const { return has_window; }
 
-void file_buffer_t::invalidate_highlight(rewrap_type_t type, int line, int pos) {
+void file_buffer_t::invalidate_highlight(rewrap_type_t type, text_pos_t line, text_pos_t pos) {
   (void)type;
   (void)pos;
   if (line <= highlight_valid) {
@@ -449,7 +443,7 @@ void file_buffer_t::set_highlight(t3_highlight_t *highlight) {
 
 bool file_buffer_t::get_strip_spaces() const {
   if (strip_spaces.is_valid()) {
-    return strip_spaces;
+    return strip_spaces.value();
   }
   return option.strip_spaces;
 }
@@ -459,8 +453,7 @@ void file_buffer_t::set_strip_spaces(bool _strip_spaces) { strip_spaces = _strip
 void file_buffer_t::do_strip_spaces() {
   size_t idx, strip_start;
   bool undo_started = false;
-  text_coordinate_t saved_cursor = cursor;
-  int i;
+  const text_coordinate_t saved_cursor = get_cursor();
 
   /*FIXME: a better way to do this would be to store the stripped spaces for
      all lines in a single string, delimited by single non-space bytes. If the
@@ -468,24 +461,24 @@ void file_buffer_t::do_strip_spaces() {
      it. Care has to be taken to maintain the correct cursor position, and
      we have to implement undo/redo separately. */
 
-  for (i = 0; i < size(); i++) {
-    const text_line_t *line = get_line_data(i);
-    const std::string *str = line->get_data();
-    const char *data = str->data();
-    strip_start = str->size();
+  for (text_pos_t i = 0; i < size(); i++) {
+    const text_line_t &line = get_line_data(i);
+    const std::string &str = line.get_data();
+    const char *data = str.data();
+    strip_start = str.size();
     for (idx = strip_start; idx > 0; idx--) {
       if ((data[idx - 1] & 0xC0) == 0x80) {
         continue;
       }
 
-      if (!line->is_space(idx - 1)) {
+      if (!line.is_space(idx - 1)) {
         break;
       }
 
       strip_start = idx - 1;
     }
 
-    if (strip_start != str->size()) {
+    if (strip_start != str.size()) {
       text_coordinate_t start, end;
       if (!undo_started) {
         start_undo_block();
@@ -493,11 +486,12 @@ void file_buffer_t::do_strip_spaces() {
       }
       start.line = end.line = i;
       start.pos = strip_start;
-      end.pos = str->size();
-      delete_block_internal(start, end, get_undo(UNDO_DELETE_BLOCK, start, end));
+      end.pos = str.size();
+      delete_block(start, end);
 
+      const text_coordinate_t cursor = get_cursor();
       if (cursor.line == i && static_cast<size_t>(cursor.pos) > strip_start) {
-        cursor.pos = strip_start;
+        set_cursor_pos(strip_start);
       }
     }
   }
@@ -506,18 +500,19 @@ void file_buffer_t::do_strip_spaces() {
     end_undo_block();
   }
 
-  cursor = saved_cursor;
-  if (cursor.pos > get_line_max(cursor.line)) {
-    cursor.pos = get_line_max(cursor.line);
+  set_cursor(saved_cursor);
+  if (saved_cursor.pos > get_line_size(saved_cursor.line)) {
+    set_cursor_pos(get_line_size(saved_cursor.line));
   }
 }
 
 bool file_buffer_t::find_matching_brace(text_coordinate_t &match_location) {
+  const text_coordinate_t cursor = get_cursor();
   file_line_t *line = static_cast<file_line_t *>(get_mutable_line_data(cursor.line));
-  char c = (*(line->get_data()))[cursor.pos];
+  char c = line->get_data()[cursor.pos];
   char c_close, check_c;
   bool forward;
-  int current_line, i, count;
+  int count;
 
   /* In the ASCII/Unicode table, the parenthesis are next to each other but
      the other braces are two apart. This is handled by using a switch with a
@@ -551,8 +546,8 @@ bool file_buffer_t::find_matching_brace(text_coordinate_t &match_location) {
   }
 
   if (forward) {
-    current_line = cursor.line;
-    i = cursor.pos;
+    text_pos_t current_line = cursor.line;
+    text_pos_t i = cursor.pos;
     count = 0;
     /* Use goto here to jump into the loop. The reason for doing this, is
        because we want to start from the location of the cursor, not the
@@ -562,9 +557,9 @@ bool file_buffer_t::find_matching_brace(text_coordinate_t &match_location) {
     for (; current_line < size(); current_line++) {
       line = static_cast<file_line_t *>(get_mutable_line_data(current_line));
       prepare_paint_line(current_line);
-      for (i = 0; i < line->get_length(); i = line->adjust_position(i, 1)) {
+      for (i = 0; i < line->size(); i = line->adjust_position(i, 1)) {
       start_search:
-        check_c = (*(line->get_data()))[i];
+        check_c = line->get_data()[i];
         if ((check_c != c && check_c != c_close) || line->get_highlight_idx(i) > 0) {
           continue;
         }
@@ -582,8 +577,8 @@ bool file_buffer_t::find_matching_brace(text_coordinate_t &match_location) {
     }
   } else {
     int open_surplus = 0;
-    int match_max, marked_pos = -1;
-    current_line = cursor.line;
+    text_pos_t match_max, marked_pos = -1;
+    text_pos_t current_line = cursor.line;
     count = -1;
 
     /* Finding the closing brace would be easy, if not for the fact that the
@@ -600,9 +595,8 @@ bool file_buffer_t::find_matching_brace(text_coordinate_t &match_location) {
        opening brace is on this line (because we count up to, but not
        including, the closing brace here).
     */
-    const int max = std::max(cursor.pos, line->get_length());
-    for (i = 0; i < max; i = line->adjust_position(i, 1)) {
-      check_c = (*(line->get_data()))[i];
+    for (text_pos_t i = 0; i < cursor.pos; i = line->adjust_position(i, 1)) {
+      check_c = line->get_data()[i];
       if ((check_c != c && check_c != c_close) || line->get_highlight_idx(i) > 0) {
         continue;
       }
@@ -630,8 +624,9 @@ bool file_buffer_t::find_matching_brace(text_coordinate_t &match_location) {
       for (current_line--; current_line >= 0; current_line--) {
         line = static_cast<file_line_t *>(get_mutable_line_data(current_line));
         /* No need to call prepare_paint_line here because we're going backwards. */
-        for (i = 0, local_count = 0, open_surplus = 0; i < line->get_length(); i++) {
-          check_c = (*(line->get_data()))[i];
+        text_pos_t i;
+        for (i = 0, local_count = 0, open_surplus = 0; i < line->size(); i++) {
+          check_c = line->get_data()[i];
           if ((check_c != c && check_c != c_close) || line->get_highlight_idx(i) > 0) {
             continue;
           }
@@ -656,7 +651,7 @@ bool file_buffer_t::find_matching_brace(text_coordinate_t &match_location) {
       if (current_line < 0) {
         return false;
       }
-      match_max = line->get_length();
+      match_max = line->size();
     } else {
       match_max = cursor.pos;
     }
@@ -673,8 +668,8 @@ bool file_buffer_t::find_matching_brace(text_coordinate_t &match_location) {
        different line.
     */
     count = -count;
-    for (i = 0; i < match_max; i = line->adjust_position(i, 1)) {
-      check_c = (*(line->get_data()))[i];
+    for (text_pos_t i = 0; i < match_max; i = line->adjust_position(i, 1)) {
+      check_c = line->get_data()[i];
       if ((check_c != c && check_c != c_close) || line->get_highlight_idx(i) > 0) {
         continue;
       }
@@ -701,7 +696,7 @@ bool file_buffer_t::find_matching_brace(text_coordinate_t &match_location) {
 bool file_buffer_t::goto_matching_brace() {
   text_coordinate_t match_coordinate;
   if (find_matching_brace(match_coordinate)) {
-    cursor = match_coordinate;
+    set_cursor(match_coordinate);
     return true;
   }
   return false;
@@ -725,14 +720,14 @@ void file_buffer_t::set_line_comment(const char *text) {
   }
 }
 
-int starts_with_comment(const std::string *text, const std::string *line_comment) {
+text_pos_t starts_with_comment(const std::string &text, const std::string &line_comment) {
   size_t i;
-  for (i = 0; i < text->size(); i++) {
-    if (text->at(i) != ' ' && text->at(i) != '\t') {
+  for (i = 0; i < text.size(); i++) {
+    if (text.at(i) != ' ' && text.at(i) != '\t') {
       break;
     }
   }
-  if (text->compare(i, line_comment->size(), *line_comment) == 0) {
+  if (text.compare(i, line_comment.size(), line_comment) == 0) {
     return i;
   }
   return -1;
@@ -744,33 +739,32 @@ void file_buffer_t::toggle_line_comment() {
   }
 
   if (get_selection_mode() == selection_mode_t::NONE) {
-    const std::string *text = get_line_data(cursor.line)->get_data();
-    int comment_start = starts_with_comment(text, &line_comment);
+    const text_coordinate_t saved_cursor = get_cursor();
+    text_pos_t comment_start =
+        starts_with_comment(get_line_data(saved_cursor.line).get_data(), line_comment);
     if (comment_start >= 0) {
-      text_coordinate_t saved_cursor = cursor;
-      delete_block(text_coordinate_t(cursor.line, comment_start),
-                   text_coordinate_t(cursor.line, comment_start + line_comment.size()));
-      if (comment_start < saved_cursor.pos) {
-        saved_cursor.pos -= std::min<int>(line_comment.size(), saved_cursor.pos - comment_start);
+      delete_block(text_coordinate_t(saved_cursor.line, comment_start),
+                   text_coordinate_t(saved_cursor.line, comment_start + line_comment.size()));
+      text_pos_t new_pos = saved_cursor.pos;
+      if (comment_start < new_pos) {
+        new_pos -= std::min<text_pos_t>(line_comment.size(), new_pos - comment_start);
       }
-      cursor.pos = saved_cursor.pos;
+      set_cursor_pos(new_pos);
     } else {
-      text_coordinate_t saved_cursor = cursor;
       // FIXME: this causes the cursor position to be recorded incorrectly in the undo information.
       // although one could argue this is to some extent better as it shows the actual edit.
-      cursor.pos = 0;
-      insert_block(&line_comment);
-      cursor.pos = saved_cursor.pos + line_comment.size();
+      set_cursor_pos(0);
+      insert_block(line_comment);
+      set_cursor_pos(saved_cursor.pos + line_comment.size());
     }
   } else {
     text_coordinate_t selection_start = get_selection_start();
     text_coordinate_t selection_end = get_selection_end();
-    int first_line = std::min(selection_start.line, selection_end.line);
-    int last_line = std::max(selection_start.line, selection_end.line);
-    int i;
+    text_pos_t first_line = std::min(selection_start.line, selection_end.line);
+    text_pos_t last_line = std::max(selection_start.line, selection_end.line);
+    text_pos_t i;
     for (i = first_line; i <= last_line; i++) {
-      const std::string *text = get_line_data(i)->get_data();
-      int comment_start = starts_with_comment(text, &line_comment);
+      text_pos_t comment_start = starts_with_comment(get_line_data(i).get_data(), line_comment);
       if (comment_start < 0) {
         break;
       }
@@ -780,19 +774,17 @@ void file_buffer_t::toggle_line_comment() {
     // for undos is as expected. Ideally we'd just call start_undo_block here.
     if (i > last_line) {
       for (i = first_line; i <= last_line; i++) {
-        const std::string *text = get_line_data(i)->get_data();
-        int comment_start = starts_with_comment(text, &line_comment);
+        text_pos_t comment_start = starts_with_comment(get_line_data(i).get_data(), line_comment);
         if (i == selection_start.line && comment_start < selection_start.pos) {
           selection_start.pos -=
-              std::min<int>(line_comment.size(), selection_start.pos - comment_start);
+              std::min<text_pos_t>(line_comment.size(), selection_start.pos - comment_start);
         }
         if (i == selection_end.line && comment_start < selection_end.pos) {
           selection_end.pos -=
-              std::min<int>(line_comment.size(), selection_end.pos - comment_start);
+              std::min<text_pos_t>(line_comment.size(), selection_end.pos - comment_start);
         }
         if (i == first_line) {
-          cursor.line = i;
-          cursor.pos = comment_start + line_comment.size();
+          set_cursor({i, comment_start + static_cast<text_pos_t>(line_comment.size())});
           start_undo_block();
         }
         delete_block(text_coordinate_t(i, comment_start),
@@ -800,35 +792,35 @@ void file_buffer_t::toggle_line_comment() {
       }
     } else {
       for (i = first_line; i <= last_line; i++) {
-        cursor.line = i;
-        cursor.pos = 0;
+        set_cursor({i, 0});
         if (i == first_line) {
           start_undo_block();
         }
-        insert_block(&line_comment);
+        insert_block(line_comment);
       }
       selection_start.pos += line_comment.size();
       selection_end.pos += line_comment.size();
     }
     end_undo_block();
     set_selection_mode(selection_mode_t::NONE);
-    cursor = selection_start;
+    set_cursor(selection_start);
     set_selection_mode(old_mode);
-    cursor = selection_end;
+    set_cursor(selection_end);
     set_selection_end();
   }
 }
 
 const char *file_buffer_t::get_char_under_cursor(size_t *size) const {
-  const text_line_t *line = get_line_data(cursor.line);
-  if (cursor.pos >= line->get_length()) {
+  const text_coordinate_t cursor = get_cursor();
+  const text_line_t &line = get_line_data(cursor.line);
+  if (cursor.pos >= line.size()) {
     *size = 1;
     return "\n";
   }
-  int end = line->adjust_position(cursor.pos, 1);
+  int end = line.adjust_position(cursor.pos, 1);
   if (cursor.pos == end) {
     return nullptr;
   }
   *size = end - cursor.pos;
-  return line->get_data()->data() + cursor.pos;
+  return line.get_data().data() + cursor.pos;
 }
