@@ -19,7 +19,7 @@
 #include <t3highlight/highlight.h>
 #include <t3widget/widget.h>
 
-using namespace t3_widget;
+using namespace t3widget;
 
 #include "tilde/filestate.h"
 
@@ -34,23 +34,23 @@ class file_buffer_t : public text_buffer_t {
   text_line_t name_line;
   std::unique_ptr<edit_window_t::view_parameters_t> view_parameters;
   bool has_window;
-  int highlight_valid;
+  text_pos_t highlight_valid;
   optional<bool> strip_spaces;
   t3_highlight_t *highlight_info;
-  text_line_t *match_line;
+  const text_line_t *match_line;
   t3_highlight_match_t *last_match;
   bool matching_brace_valid;
   text_coordinate_t matching_brace_coordinate;
   std::string line_comment;
 
  private:
-  void prepare_paint_line(int line) override;
+  void prepare_paint_line(text_pos_t line) override;
   void set_has_window(bool _has_window);
-  void invalidate_highlight(rewrap_type_t type, int line, int pos);
+  void invalidate_highlight(rewrap_type_t type, text_pos_t line, text_pos_t pos);
   bool find_matching_brace(text_coordinate_t &match_location);
 
  public:
-  explicit file_buffer_t(const char *_name = nullptr, const char *_encoding = nullptr);
+  explicit file_buffer_t(string_view _name = {"", 0}, string_view _encoding = {"", 0});
   ~file_buffer_t() override;
   rw_result_t load(load_process_t *state);
   rw_result_t save(save_as_process_t *state);
