@@ -17,18 +17,18 @@
 #include "tilde/filebuffer.h"
 #include <t3widget/widget.h>
 
-using namespace t3_widget;
+using namespace t3widget;
 
 class file_edit_window_t : public edit_window_t {
  private:
-  signals::connection rewrap_connection;
-  void force_repaint_to_bottom(rewrap_type_t type, int line, int pos);
+  connection_t rewrap_connection;
+  void force_repaint_to_bottom(rewrap_type_t type, text_pos_t line, text_pos_t pos);
 
  public:
   explicit file_edit_window_t(file_buffer_t *_text = nullptr);
   ~file_edit_window_t() override;
   void draw_info_window() override;
-  bool process_key(t3_widget::key_t key) override;
+  bool process_key(t3widget::key_t key) override;
   void update_contents() override;
 
   void set_text(file_buffer_t *_text);
