@@ -20,28 +20,19 @@ file_line_t::file_line_t(int buffersize, file_line_factory_t *_factory)
     : text_line_t(buffersize, _factory == nullptr ? &default_file_line_factory : _factory),
       highlight_start_state(0) {}
 
-file_line_t::file_line_t(const char *_buffer, file_line_factory_t *_factory)
+file_line_t::file_line_t(string_view _buffer, file_line_factory_t *_factory)
     : text_line_t(_buffer, _factory == nullptr ? &default_file_line_factory : _factory),
       highlight_start_state(0) {}
 
-file_line_t::file_line_t(const char *_buffer, int length, file_line_factory_t *_factory)
-    : text_line_t(_buffer, length, _factory == nullptr ? &default_file_line_factory : _factory),
-      highlight_start_state(0) {}
-
-file_line_t::file_line_t(const std::string *str, file_line_factory_t *_factory)
-    : text_line_t(str, _factory == nullptr ? &default_file_line_factory : _factory),
-      highlight_start_state(0) {}
-
-int file_line_t::get_highlight_idx(int i) {
-  const std::string *str;
-  file_buffer_t *file = static_cast<file_line_factory_t *>(factory)->get_file_buffer();
+int file_line_t::get_highlight_idx(text_pos_t i) const {
+  file_buffer_t *file = static_cast<file_line_factory_t *>(get_line_factory())->get_file_buffer();
 
   if (file == nullptr || file->highlight_info == nullptr) {
     return -1;
   }
 
-  str = get_data();
-  if (static_cast<size_t>(i) >= str->size()) {
+  const std::string &str = get_data();
+  if (static_cast<size_t>(i) >= str.size()) {
     return -1;
   }
 
@@ -52,7 +43,7 @@ int file_line_t::get_highlight_idx(int i) {
   }
 
   while (t3_highlight_get_end(file->last_match) <= static_cast<size_t>(i)) {
-    t3_highlight_match(file->last_match, str->data(), str->size());
+    t3_highlight_match(file->last_match, str.data(), str.size());
   }
 
   return static_cast<size_t>(i) < t3_highlight_get_match_start(file->last_match)
@@ -60,14 +51,14 @@ int file_line_t::get_highlight_idx(int i) {
              : t3_highlight_get_match_attr(file->last_match);
 }
 
-t3_attr_t file_line_t::get_base_attr(int i, const paint_info_t *info) {
-  file_buffer_t *file = static_cast<file_line_factory_t *>(factory)->get_file_buffer();
+t3_attr_t file_line_t::get_base_attr(text_pos_t i, const paint_info_t &info) const {
+  file_buffer_t *file = static_cast<file_line_factory_t *>(get_line_factory())->get_file_buffer();
   int idx = get_highlight_idx(i);
-  t3_attr_t result = idx < 0 ? info->normal_attr : option.highlights[idx];
+  t3_attr_t result = idx < 0 ? info.normal_attr : option.highlights[idx];
 
   if (file->matching_brace_valid &&
-      (i == info->cursor || (this == file->get_line_data(file->matching_brace_coordinate.line) &&
-                             i == file->matching_brace_coordinate.pos))) {
+      (i == info.cursor || (this == &file->get_line_data(file->matching_brace_coordinate.line) &&
+                            i == file->matching_brace_coordinate.pos))) {
     result = t3_term_combine_attrs(result, option.brace_highlight);
   }
 
@@ -77,9 +68,7 @@ t3_attr_t file_line_t::get_base_attr(int i, const paint_info_t *info) {
 void file_line_t::set_highlight_start(int state) { highlight_start_state = state; }
 
 int file_line_t::get_highlight_end() {
-  const std::string *str;
-
-  file_buffer_t *file = static_cast<file_line_factory_t *>(factory)->get_file_buffer();
+  file_buffer_t *file = static_cast<file_line_factory_t *>(get_line_factory())->get_file_buffer();
   if (file == nullptr || file->highlight_info == nullptr) {
     return 0;
   }
@@ -89,8 +78,8 @@ int file_line_t::get_highlight_end() {
     t3_highlight_reset(file->last_match, highlight_start_state);
   }
 
-  str = get_data();
-  while (t3_highlight_match(file->last_match, str->data(), str->size())) {
+  const std::string &str = get_data();
+  while (t3_highlight_match(file->last_match, str.data(), str.size())) {
   }
 
   return t3_highlight_get_state(file->last_match);
@@ -102,20 +91,12 @@ file_line_factory_t::file_line_factory_t(file_buffer_t *_file_buffer) {
   file_buffer = _file_buffer;
 }
 
-text_line_t *file_line_factory_t::new_text_line_t(int buffersize) {
-  return new file_line_t(buffersize, this);
+std::unique_ptr<text_line_t> file_line_factory_t::new_text_line_t(int buffersize) {
+  return t3widget::make_unique<file_line_t>(buffersize, this);
 }
 
-text_line_t *file_line_factory_t::new_text_line_t(const char *_buffer) {
-  return new file_line_t(_buffer, this);
-}
-
-text_line_t *file_line_factory_t::new_text_line_t(const char *_buffer, int length) {
-  return new file_line_t(_buffer, length, this);
-}
-
-text_line_t *file_line_factory_t::new_text_line_t(const std::string *str) {
-  return new file_line_t(str, this);
+std::unique_ptr<text_line_t> file_line_factory_t::new_text_line_t(string_view _buffer) {
+  return t3widget::make_unique<file_line_t>(_buffer, this);
 }
 
 file_buffer_t *file_line_factory_t::get_file_buffer() const { return file_buffer; }
